@@ -12,10 +12,10 @@ import numpy as np
 
 # vivarium core imports
 from vivarium.library.schema import array_from, array_to
-from vivarium.core.experiment import pp
-from vivarium.core.process import Composite
+from vivarium.core.engine import pp
+from vivarium.core.composer import Composer
 from vivarium.core.composition import (
-    simulate_compartment_in_experiment,
+    simulate_composer,
     COMPARTMENT_OUT_DIR,
 )
 from vivarium.plots.simulation_output import plot_simulation_output
@@ -29,7 +29,7 @@ from vivarium_bioscrape.processes.one_way_map import OneWayMap
 NAME = 'bioscrape_connector'
 
 
-class BioscrapeConnector(Composite):
+class BioscrapeConnector(Composer):
     name = NAME
     defaults = {
         'models': {},
@@ -52,8 +52,7 @@ class BioscrapeConnector(Composite):
         self.models = {}
         self.connections = {}
         self.connection_counts = {} #Stores the number of connections between pairs of processes, for naming purposes
-
-        super(BioscrapeConnector, self).__init__(config)
+        super().__init__(config)
         self.topology = self.initial_topology(self.config)
 
     def initial_state(self, config=None):
@@ -83,20 +82,23 @@ class BioscrapeConnector(Composite):
             target_deltas = map_function(modified_source_state)
 
             for species, value in target_deltas.items():
-                target_value = target_state['species'][species]
-                if value != 0.0 and target_value != value:
-                    raise ValueError(
-                        f"initial condition ['{target}_species']['{species}']={target_value} "
-                        f"does not match mapping from model {source} to model {target}")
+                 target_value = target_state['species'][species]
+                 if value != 0.0 and target_value != value:
+                     raise ValueError(
+                         f"initial condition ['{target}_species']['{species}']={target_value} "
+                         f"does not match mapping from model {source} to model {target}")
 
             # set initial state according source and target species
             initial_state[f'{source}_species'] = source_state['species']
             initial_state[f'{target}_species'] = target_state['species']
 
+        #Reset internal variables because generate_processes will be called again
         return initial_state
 
     def generate_processes(self, config):
         # make bioscrape processes
+        
+        self.connection_counts = {} #models and connections may be overwritten, so reset the counts
         for name, parameters in config['models'].items():
             self.add_model(name = name, bioscrape_parameters = parameters)
 
@@ -107,6 +109,7 @@ class BioscrapeConnector(Composite):
             self.add_connection(source, target, map_function)
 
         # combine model and connection processes
+
         return {**self.models, **self.connections}
 
     def generate_topology(self, config):
@@ -128,6 +131,14 @@ class BioscrapeConnector(Composite):
             source = connection['source']
             target = connection['target']
 
+            #Count is used to allow multiple connections between models if desired
+            if (source, target) in self.connection_counts:
+                self.connection_counts[(source, target)] += 1
+                count = self.connection_counts[(source, target)]
+            else:
+                count = 1
+                self.connection_counts[(source, target)] = count
+
             connections[f'{source}_{target}_connector_{count}'] = {
                 'source_deltas': (f'{source}_deltas',),
                 'target_state': (f'{target}_species',),
@@ -144,8 +155,9 @@ class BioscrapeConnector(Composite):
         if name is None:
             name = str(len(self.models))
 
-        if name in self.models:
-            raise ValueError(f"A model named {name} already exists!")
+        #This test doesn't allow generate to be called multiple times
+        #if name in self.models:
+        #    raise ValueError(f"A model named {name} already exists!")
 
         if bioscrape_process is not None and bioscrape_parameters is not None:
             raise ValueError("Recieved both a bioscrape_process and bioscrape_parameters! Please use one or the other.")
@@ -207,15 +219,8 @@ class BioscrapeConnector(Composite):
             }
         }
 
-
-def main():
-    '''Simulate the composite and plot results.'''
-
-    # make an output directory to save plots
-    out_dir = os.path.join(COMPARTMENT_OUT_DIR, NAME)
-    if not os.path.exists(out_dir):
-        os.makedirs(out_dir)
-
+def test_connector():
+    
     bioscrape_process_1 = Bioscrape(parameters = {
         'sbml_file': 'Notebooks/model1.xml'
         })
@@ -236,7 +241,7 @@ def main():
         model3_vector/np.sum(model3_vector),
         model1_vector/np.sum(model1_vector))
 
-    # define map function
+        # define map function
     def map_1_3(states):
         input_array = array_from(states['source_deltas'])
         output_array = np.dot(input_array, projection_1_3)
@@ -262,12 +267,12 @@ def main():
         {'source': '1', 'target': '3', 'map_function': map_1_3},
         {'source': '3', 'target': '1', 'map_function': map_3_1}
     ]
-
     # make the composite
     composite = BioscrapeConnector(
         models=models,
         connections=connections,
     )
+    
 
     ## Run a simulation
     # initial state
@@ -278,11 +283,38 @@ def main():
     }
     initial_state = composite.initial_state(config)
 
+    #This occurs after composite.generate() is called
+    assert len(composite.models) == 2
+    assert len(composite.connections) == 2
+
     # run a simulation
     sim_settings = {
-        'total_time': 500,
+        'total_time': 10,
         'initial_state': initial_state}
-    output = simulate_compartment_in_experiment(composite, sim_settings)
+    output = simulate_composer(composite, sim_settings)
+
+    #DNA should be constant
+    assert all([output['1_species']['dna_G'][0] == g for g in output['1_species']['dna_G']])
+
+    #RNA should be the same between the two simulations
+    assert output['1_species']['rna_T'] == output['3_species']['rna_T']
+
+    #total RNAase should be constant
+    RNAase_tot = output['3_species']['protein_RNAase'][0]
+
+    rnas = [output['3_species']['protein_RNAase'][i]+output['3_species']['complex_protein_RNAase_rna_T_'][i] for i in range(len(output['3_species']['complex_protein_RNAase_rna_T_']))]
+
+    #Slight numerical errors are possible, but total RNAase should be nearly constant
+    assert  all([np.abs(s-RNAase_tot) < 10**-6 for s in rnas])
+
+def main():
+    '''Simulate the composite and plot results.'''
+    #make an output directory to save plots
+    out_dir = os.path.join(COMPARTMENT_OUT_DIR, NAME)
+    if not os.path.exists(out_dir):
+        os.makedirs(out_dir)
+
+    output = test_connector()
 
     # plot simulation output
     plot_settings = {}
